#include "MainApplication.hpp"

extern lang::Language *language;

namespace ui
{
    extern MainApplication *mainapp;

    AmiiboDetailsLayout::AmiiboDetailsLayout() : pu::Layout()
    {
        this->amiiboImage = new pu::element::Image(10, 90, utils::GetRomFsResource("Common/logo.png"));
        this->amiiboImage->SetWidth(200);
        this->amiiboImage->SetHeight(200);
        this->Add(this->amiiboImage);

        this->amiiboName = new pu::element::TextBlock(220, 260, "", 30);
        this->Add(this->amiiboName);

        this->firstWrite = new pu::element::TextBlock(10, 310, "", 20);
        this->Add(this->firstWrite);

        this->lastWrite = new pu::element::TextBlock(10, 350, "", 20);
        this->Add(this->lastWrite);

        this->amiiboPath = new pu::element::TextBlock(10, 390, "", 20);
        this->Add(this->amiiboPath);

        this->amiiboRandom = new pu::element::TextBlock(10, 430, "", 20);
        this->Add(this->amiiboRandom);

        this->SetOnInput([&](u64 Down, u64 Up, u64 Held, bool Touch)
        {
            if (Down & KEY_B){
                mainapp->SetHelpText(lang::GetLabel(lang::Label::HELP_SELECT));
                mainapp->GetMainLayout()->GetMainMenu()->SetVisible(true);
                mainapp->GetMainLayout()->SetElementOnFocus(mainapp->GetMainLayout()->GetMainMenu());
                mainapp->GetMainLayout()->selectionChange();
                mainapp->LoadLayout(mainapp->GetMainLayout());
            }
        });
    }

    AmiiboDetailsLayout::~AmiiboDetailsLayout()
    {
        delete this->amiiboImage;
        delete this->amiiboName;
        delete this->amiiboPath;
        delete this->firstWrite;
        delete this->lastWrite;
    }

    void AmiiboDetailsLayout::LoadAmiibo(std::string path)
    {
        std::string imagePath = path + "/amiibo.icon";
        this->amiiboImage->SetImage(imagePath);
        
        std::string jsonPath = path + "/tag.json";
        std::ifstream ifs(jsonPath);
        auto amiiboJson = json::parse(ifs);
        if(ifs.is_open())
            ifs.close();
        
        std::string jsonPath2 = path + "/register.json";
        std::ifstream ofs(jsonPath2);
        auto amiiboJson2 = json::parse(ofs);
        if(ofs.is_open())
            ofs.close();
            
        std::string jsonPath3 = path + "/common.json";
        std::ifstream pfs(jsonPath3);
        auto amiiboJson3 = json::parse(pfs);
        if(pfs.is_open())
            pfs.close();
        
        this->randomuuid = amiiboJson.value<bool>("randomUuid", false);
        
        if(randomuuid)
            this->amiiboRandom->SetText(lang::GetLabel(lang::Label::RANDOM_UUID) + lang::GetLabel(lang::Label::ENABLED));
        else
            this->amiiboRandom->SetText(lang::GetLabel(lang::Label::RANDOM_UUID) + lang::GetLabel(lang::Label::DISABLED));
        		this->amiiboName->SetText((std::string)amiiboJson2["name"].get<std::string>());
        		this->amiiboPath->SetText(lang::GetLabel(lang::Label::AMIIBO_DETAILS_PATH) + path);
        		this->firstWrite->SetText(lang::GetLabel(lang::Label::AMIIBO_DETAILS_FIRST_DATE) + std::to_string((u16)amiiboJson2["firstWriteDate"][0].get<int>()) + "/" + std::to_string((u8)amiiboJson2["firstWriteDate"][1].get<int>()) + "/" + std::to_string((u8)amiiboJson2["firstWriteDate"][2].get<int>()));
        		this->lastWrite->SetText(lang::GetLabel(lang::Label::AMIIBO_DETAILS_LAST_DATE) + std::to_string((u16)amiiboJson3["lastWriteDate"][0].get<int>()) + "/" + std::to_string((u8)amiiboJson3["lastWriteDate"][1].get<int>()) + "/" + std::to_string((u8)amiiboJson3["lastWriteDate"][2].get<int>()));
    }
}
